/*

Copyright (c) 2008,2009 SFB/TR-55
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef LIBROC_H_
#define LIBROC_H_

#include <stdint.h>

/** foursh default port.
 * The foursh listens on this port by default.
 */
#define ROC_FOURSH_PORT 30444

/** Scripting routine for the qpace rootcard.
 *
 * This routine allows the remote executon of commands on
 * the qpace rootcard via the foursh.
 * Commands on the rootcard can be mixed with commands
 * on the frontend.
 *
 * @param ip Null-terminated string in dot format "aaa.bbb.ccc.ddd"
 *      representing an IP4 address
 * @param port Port to connect to.
 *      Usually you want to use @a ROC_FOURSH_PORT.
 * @param target_fd The reply of the foush is written to this
 *      file descriptor. Pass @c -1 if you don't want to
 *      pass the data to any file descriptor.
 * @param buflen The reply can also be written to a buffer.
 *      This parameter is the length of this buffer.
 *      Pass @c -1 if you don't want to use this feature.
 * @param buffer The buffer. Pass @c NULL if you don't
 *      want to use this feature.
 * @param ac number of elements in parameter @a av
 * @param av array of null-terminated strings.
 *      Strings starting with one '\@' character are
 *      executed on the frontend via the @c system call,
 *      other strings are passed to the foursh.
 *      A newline is automatically appended.
 *
 *      String starting with two at-characters (\@\@) are
 *      special commands that change the behavior of this
 *      function. Currently two special commands are implemented:
 *
 *      \@\@fgets-stdin will cause this function to contine reading
 *      lines from stding via fgets() after all times in @a av have
 *      been consumed,
 *
 *      \@\@no-fgets-stdin will cause this function to stop reading
 *      from stdin.
 */
int
rocMainScript(
    const char* ip, 
    uint16_t port, 
    int target_fd, 
    int buflen,
    char* buffer,
    int ac, char** av);


/** Scripting routine for the qpace rootcard.
 *
 * This routine allows the remote executon of commands on
 * the qpace rootcard via the foursh.
 * Commands on the rootcard can be mixed with commands
 * on the frontend.
 *
 * @param ip Null-terminated string in dot format "aaa.bbb.ccc.ddd"
 *      representing an IP4 address
 * @param port Port to connect to.
 *      Usually you want to use @a ROC_FOURSH_PORT.
 * @param target_fd The reply of the foush is written to this
 *      file descriptor. Pass @c -1 if you don't want to
 *      pass the data to any file descriptor.
 * @param buflen The reply can also be written to a buffer.
 *      This parameter is the length of this buffer.
 *      Pass @c -1 if you don't want to use this feature.
 * @param buffer The buffer. Pass @c NULL if you don't
 *      want to use this feature.
 * @param ac number elements to follow.
 *
 * Unlike @a rocMainScript, the number of arguments is variable.
 * instead of passing an array of type @c char*, just pass
 * the list of strings.
 */
int
rocMainScriptAV(
    const char* ip,
    uint16_t port,
    int target_fd,
    int buflen,
    char* buffer,
    int ac,
    ...);

#endif

